// TODO: akamaru plugin

#ifdef USE_AKAMARU
static gint kiba_dock_akamaru_timeout (gpointer data) {
	GtkWidget *widget = data;
	KibaDockPrivate *priv;
	GList *l;
	
	if (!kiba_dock_akamaru_enabled(KIBA_DOCK(widget)))
		return FALSE;
	
	priv = KIBA_DOCK_GET_PRIVATE (widget);
	
	/* calculate new object positions */
	kiba_akamaru_model_step (priv->model);
	
	for (l = GTK_BOX(widget)->children; l != NULL; l = l->next) {
		GtkBoxChild *child = l->data;
		KibaAkamaruObject *obj;
		gint width, height;
		
		if (KIBA_IS_FIXED_OBJECT(child->widget))
			continue;
		
		if (!GTK_IS_WINDOW(KIBA_OBJECT(child->widget)->akamaru_win))
			continue;
		obj = g_object_get_data (G_OBJECT(child->widget), "akamaru-object");
		width = child->widget->allocation.width;
		height = child->widget->allocation.height;
		
		gtk_widget_set_usize (KIBA_OBJECT(child->widget)->akamaru_win,
			width,
			height);
		
		gtk_window_move (GTK_WINDOW(KIBA_OBJECT(child->widget)->akamaru_win),
		       obj->object->position.x + 0.5 - child->widget->allocation.width/2,
		       obj->object->position.y + 0.5 - child->widget->allocation.width/2);

		child->widget->allocation.x = obj->object->position.x + 0.5 - child->widget->allocation.width/2;
		child->widget->allocation.y = obj->object->position.y + 0.5 - child->widget->allocation.width/2;
	}
	
	return TRUE;
}

static void kiba_dock_hidden_cb (KibaWin  *win,
				 KibaDock *dock) {
	GList *l;
	
	for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
		GtkBoxChild *child = l->data;
		
		if (!GTK_IS_WINDOW(KIBA_OBJECT(child->widget)->akamaru_win))
			continue;
		
		gtk_widget_hide (KIBA_OBJECT(child->widget)->akamaru_win);
	}
}

static void kiba_dock_unhidden_cb (KibaWin  *win,
				   KibaDock *dock) {
	GList *l;
	
	for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
		GtkBoxChild *child = l->data;
		
		if (!GTK_IS_WINDOW(KIBA_OBJECT(child->widget)->akamaru_win))
			continue;
		
		gtk_widget_show (KIBA_OBJECT(child->widget)->akamaru_win);
	}
}

static gboolean kiba_dock_button_press_event_cb (GtkWidget      *widget,
					         GdkEventButton *event) {
	if (kiba_dock_akamaru_enabled(KIBA_DOCK(widget))) {
		GList *l;
		
		for (l = GTK_BOX(widget)->children; l != NULL; l = l->next) {
			GtkBoxChild *child = l->data;
			KibaAkamaruObject *obj;
			gint width, height;
			
			if (KIBA_IS_FIXED_OBJECT(child->widget))
				continue;
			
			if (!GTK_IS_WINDOW(KIBA_OBJECT(child->widget)->akamaru_win))
				continue;
			
			gtk_window_present (GTK_WINDOW(KIBA_OBJECT(child->widget)->akamaru_win));
		}
	}
	
	return FALSE;
}

static gboolean kiba_akamaru_win_expose (GtkWidget      *widget,
					 GdkEventExpose *expose) {
	KibaObject *object = KIBA_OBJECT(widget);
	cairo_t *cr = NULL;
	
	cr = gdk_cairo_create (expose->window);
	if (cairo_status (cr) != CAIRO_STATUS_SUCCESS)
		return TRUE;
	
	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
	cairo_set_source_rgba (cr, 0, 0, 0, 0);
	cairo_paint(cr);
	
	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
	cairo_set_source_surface (cr, object->surface_data->surface, 0, 0);
	kiba_object_paint_with_prelight (object,
		object->surface_data->surface,
		cr);
	
	cairo_destroy (cr);
	
	return TRUE;
}

static GtkWidget* kiba_akamaru_object_create_window (GtkWidget         *widget,
						     KibaAkamaruObject *object) {
	GdkScreen *screen = gdk_screen_get_default ();
	GtkWidget *ret;
	GtkWidget *image;
	
	ret = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_add_events (GTK_WIDGET (ret),
		GDK_EXPOSURE_MASK |
		GDK_BUTTON_PRESS_MASK |
		GDK_BUTTON_RELEASE_MASK |
		GDK_ENTER_NOTIFY_MASK |
		GDK_LEAVE_NOTIFY_MASK |
		GDK_POINTER_MOTION_MASK |
		GDK_POINTER_MOTION_HINT_MASK);
	gtk_widget_set_app_paintable (ret, TRUE);
	gtk_window_set_decorated (GTK_WINDOW(ret), FALSE);
	gtk_window_set_resizable (GTK_WINDOW(ret), FALSE);
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW(ret), TRUE);
	gtk_window_set_type_hint (GTK_WINDOW(ret), GDK_WINDOW_TYPE_HINT_DOCK);
	gtk_widget_set_colormap (ret, gdk_screen_get_rgba_colormap (screen));
	
	gtk_widget_set_usize (ret,
		widget->allocation.width,
		widget->allocation.height);
	
	g_signal_connect (G_OBJECT(widget), "expose-event",
		G_CALLBACK(kiba_akamaru_win_expose), NULL);
	
	gtk_widget_realize(ret);
	
	gdk_window_set_user_data (ret->window, widget);
	gdk_window_set_keep_above (ret->window, TRUE);
	
	return ret;
}

static KibaAkamaruObject* kiba_dock_create_akamaru_object (KibaDock  *dock,
							   GtkWidget *widget) {
	KibaAkamaruObject *ret;
	GdkWindow *window;
	GdkRectangle gravity_normal;
	const KibaLayoutData *d;
	KibaDockPrivate *priv;
	
	if (GTK_IS_WINDOW(KIBA_OBJECT(widget)->akamaru_win))
		return;
	
	priv = KIBA_DOCK_GET_PRIVATE (dock);
	
	d = &kiba_layout_data[dock->d.position];
	
	ret = kiba_akamaru_add_object (priv->model,
		KIBA_SCREEN_DEFAULT->geometry.width,
		KIBA_SCREEN_DEFAULT->geometry.height,
		MAX(widget->allocation.width, widget->allocation.height)/2.0);
	KIBA_OBJECT(widget)->akamaru_win = kiba_akamaru_object_create_window (widget, ret);
	
	if (d->orientation == GTK_ORIENTATION_VERTICAL) {
		gravity_normal.x = (d->halignment - 1);
		gravity_normal.y = 0;
	}
	else {
		gravity_normal.x = 0;
		gravity_normal.y = (d->valignment - 1);
	}
	
	kiba_akamaru_object_set_gravity_point (priv->model,
		ret, gravity_normal);
	
	gtk_window_present (GTK_WINDOW(KIBA_OBJECT(widget)->akamaru_win));
	
	g_object_set_data (G_OBJECT(widget), "akamaru-object", ret);
	
	return ret;
}
#endif

void kiba_dock_set_akamaru_property (KibaDock   *dock,
				     const char *property,
				     gpointer    value) {
	GtkWidget *widget = GTK_WIDGET(dock);
	GList *l;
	KibaDockPrivate *priv;
	const KibaLayoutData *d;
	
	g_return_if_fail (KIBA_IS_DOCK(dock));
#ifdef USE_AKAMARU
	
	d = &kiba_layout_data[KIBA_DOCK(dock)->d.position];
	
	priv = KIBA_DOCK_GET_PRIVATE (dock);
	
	g_return_if_fail (priv->model != NULL);
	
	if (strcmp (property, "akamaru_enable") == 0) {
		if (!kiba_akamaru_is_enabled (priv->model) && GPOINTER_TO_INT (value)) {
			kiba_akamaru_model_enable (priv->model, TRUE);
			
			if (!priv->akamaru_id)
				priv->akamaru_id = g_timeout_add (20, kiba_dock_akamaru_timeout, dock);
			
			kiba_akamaru_set_anchor_position (priv->model,
				widget->allocation.x + widget->allocation.width/2,
				widget->allocation.y + widget->allocation.height/2);
			
			for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
				GtkBoxChild *child = l->data;
				KibaObject *object = KIBA_OBJECT(child->widget);
				
				if (KIBA_IS_FIXED_OBJECT(object))
					continue;
				
				gdk_window_hide (object->event_window);
				kiba_dock_create_akamaru_object (dock, child->widget);
			}
			
			gtk_widget_queue_draw (GTK_WIDGET(dock));
			
			g_signal_connect(G_OBJECT(dock), "button-press-event",
				G_CALLBACK(kiba_dock_button_press_event_cb), NULL);
			g_signal_connect(G_OBJECT(widget->parent), "dock-hidden",
				G_CALLBACK(kiba_dock_hidden_cb), NULL);
			g_signal_connect(G_OBJECT(widget->parent), "dock-unhidden",
				G_CALLBACK(kiba_dock_unhidden_cb), NULL);
		}
		else if (kiba_akamaru_is_enabled (priv->model)) {
			g_source_remove (priv->akamaru_id);
			priv->akamaru_id = 0;
			
			for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
				GtkBoxChild *child = l->data;
				KibaObject *object;
				KibaAkamaruObject *obj;
				
				if (KIBA_IS_FIXED_OBJECT(child->widget))
					continue;
				
				object = KIBA_OBJECT(child->widget);
				if (object->akamaru_win) {
					gtk_widget_destroy (object->akamaru_win);
					object->akamaru_win = NULL;
				}
				
				obj = g_object_get_data (G_OBJECT(object), "akamaru-object");
				if (obj) {
					kiba_akamaru_object_free (priv->model, obj);
					g_object_set_data (G_OBJECT(object), "akamaru-object", NULL);
				}
				gdk_window_show (object->event_window);
			}
			
			kiba_akamaru_model_enable (priv->model, FALSE);
			
			__sort_objects(dock);
			__allocate_children_size (GTK_WIDGET(dock));
// 			gtk_widget_queue_draw (GTK_WIDGET(dock));
			
			gtk_widget_queue_resize (GTK_WIDGET(dock));
			
			g_signal_handlers_disconnect_by_func (G_OBJECT(dock),
				G_CALLBACK(kiba_dock_button_press_event_cb), NULL);
			g_signal_handlers_disconnect_by_func (G_OBJECT(widget->parent),
				G_CALLBACK(kiba_dock_unhidden_cb), NULL);
			g_signal_handlers_disconnect_by_func (G_OBJECT(widget->parent),
				G_CALLBACK(kiba_dock_hidden_cb), NULL);
		}
	}
	else if (strcmp (property, "akamaru_model") == 0)
		kiba_akamaru_set_model_from_string (priv->model, (char*) value);
	else if (strcmp (property, "akamaru_elasticity") == 0)
		kiba_akamaru_set_elasticity (priv->model, atof((char*) value));
	else if (strcmp (property, "akamaru_friction") == 0)
		kiba_akamaru_set_friction (priv->model, atof((char*) value));
	else if (strcmp (property, "akamaru_k") == 0) {
		kiba_akamaru_set_k (priv->model, atof((char*) value));
	}
	else if (strcmp (property, "akamaru_gravity") == 0) {
		GdkRectangle gravity_normal;
		
		if (d->orientation == GTK_ORIENTATION_VERTICAL) {
			gravity_normal.x = (d->halignment - 1);
			gravity_normal.y = 0;
		}
		else {
			gravity_normal.x = 0;
			gravity_normal.y = (d->valignment - 1);
		}
		
		kiba_akamaru_set_gravity (priv->model, atof((char*) value));
		
		for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
			GtkBoxChild *child = l->data;
			KibaAkamaruObject *obj;
			
			obj = g_object_get_data (G_OBJECT(child->widget), "akamaru-object");
			
			if (!obj)
				continue;
			
			kiba_akamaru_object_set_gravity_point (priv->model,
				obj, gravity_normal);
		}
	}
	else if (strcmp (property, "akamaru_inertia") == 0)
		kiba_akamaru_set_inertia (priv->model, atof((char*) value));
	else if (strcmp (property, "akamaru_throw_gravity") == 0)
		kiba_akamaru_set_throw_gravity (priv->model, atof((char*) value));
	else if (strcmp (property, "akamaru_wobbly_spring") == 0)
		kiba_akamaru_set_wobbly_spring (priv->model, atof((char*) value));
	else if (strcmp (property, "akamaru_rope_string_length") == 0)
		kiba_akamaru_set_rope_string_length (priv->model, atof((char*) value));
#endif
}

/**
 * kiba_dock_akamaru_enabled:
 * @dock: The KibaDock instance.
 *
 * Returns wether akamaru is enabled.
 *
 * Returns: @c TRUE if akamaru is enabled, otherwise @c FALSE is returned.
 */
gboolean kiba_dock_akamaru_enabled (KibaDock *dock) {
#ifdef USE_AKAMARU
	KibaDockPrivate *priv;
	
	g_return_val_if_fail (KIBA_IS_DOCK(dock), FALSE);
	
	priv = KIBA_DOCK_GET_PRIVATE (dock);
	
	if (!priv->model)
		return FALSE;
	
	return kiba_akamaru_is_enabled (priv->model);
#endif
	return FALSE;
}