/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_STACK_ICON_H__
#define __KIBA_STACK_ICON_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>

#include <kiba-cairo-utils.h>

G_BEGIN_DECLS

typedef struct _KibaStackIcon         KibaStackIcon;
typedef struct _KibaStackIconClass    KibaStackIconClass;

#include "stack.h"

#define KIBA_TYPE_STACK_ICON           (kiba_stack_icon_get_type ())
#define KIBA_STACK_ICON(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_STACK_ICON, KibaStackIcon))
#define KIBA_STACK_ICON_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_STACK_ICON,  KibaStackIconClass))
#define KIBA_IS_STACK_ICON(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_STACK_ICON))
#define KIBA_IS_STACK_ICON_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_STACK_ICON))
#define KIBA_STACK_ICON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_STACK_ICON, KibaStackIconClass))

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	KIBA_STACK_STATE_ACTIVE = 0,
	KIBA_STACK_STATE_INACTIVE
}KibaStackIconState;

typedef enum {
	KIBA_STACK_ICON_TYPE_MIME_FILE = 0,
	KIBA_STACK_ICON_TYPE_THUMB_FILE,
	KIBA_STACK_ICON_TYPE_DESKTOP,
	KIBA_STACK_ICON_TYPE_DIRECTORY
}KibaStackIconType;

struct _KibaStackIcon {
	GObject                  parent;
	
	KibaStack               *stack;
	
	KibaStackIconType        type;
	KibaStackIconState       state;
	GnomeVFSMimeApplication *app;
	char                    *url;
	char                    *name;
	char                    *mime_icon;
	char                    *mime_type;
	char                    *exec;
	char                    *custom_icon;
	
	CairoSurfaceData        *surface;
};

struct _KibaStackIconClass {
	GObjectClass parent_class;
	
	void (* activate) (KibaStackIcon *icon);
	void (* deactivate) (KibaStackIcon *icon);
};

KibaStackIcon* kiba_stack_icon_new (KibaStack *stack);

gboolean kiba_stack_icon_set_url (KibaStackIcon *icon,
			          const char    *url);

void kiba_stack_icon_set_state (KibaStackIcon      *icon,
				KibaStackIconState  state);

KibaStackIconState kiba_stack_icon_get_state (KibaStackIcon *icon);

void kiba_stack_icon_update_surface (KibaStackIcon *icon);

void kiba_stack_icon_destroy (KibaStackIcon *icon);

G_END_DECLS

#endif
