/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_STACK_H
#define __KIBA_STACK_H

#include <glib.h>
#include <libgnomeui/libgnomeui.h>

#include <kiba-plugin.h>
#include <kiba-icon-view-win.h>

G_BEGIN_DECLS

typedef struct _KibaStack         KibaStack;
typedef struct _KibaStackClass    KibaStackClass;

typedef struct _KibaStackPlugin   KibaStackPlugin;

#include "stack-icon.h"

#define KIBA_TYPE_STACK           (kiba_stack_get_type ())
#define KIBA_STACK(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_STACK, KibaStack))
#define KIBA_STACK_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TYPE_STACK,  KibaStackClass))
#define KIBA_IS_STACK(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_STACK))
#define KIBA_IS_STACK_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_STACK))
#define KIBA_STACK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_STACK, KibaStackClass))

#define STACK_PLUGIN_NAME "stack"
#define STACK_DIR          "/.kiba-dock/" STACK_PLUGIN_NAME

typedef struct {
	char *mime_type;
	char *command;
}KibaStackCustomMimeApp;

typedef enum {
	KIBA_STACK_TYPE_CUSTOM = 0,
	KIBA_STACK_TYPE_DIRECTORY,
	KIBA_STACK_TYPE_NONE
}KibaViewType;

typedef struct {
	GQuark           type_quark;
	GQuark           icon_quark;
	cairo_surface_t *surface;
}KibaStackMimeSurface;

struct _KibaStackPlugin {
	KibaPlugin            *plugin;
	
	GList                 *stacks; /* list with all KibaStack's */
	
	GnomeThumbnailFactory *thumb_factory;
	
	GList                 *surface_pool; /* list with all KibaStackMimeSurface's */
	
	/** Icon-View-Win **/
	KibaIconViewWin       *view_win;
	KibaViewType           view_type;
	KibaStack             *view_stack; /* the stack the icon-view was created for */
	KibaStackIcon         *view_icon; /* the icon the icon-view was created for */
	char                  *view_path;
	
	/** Properties **/
	gint                   view_icon_size;
	gint                   view_rows;
	gint                   view_columns;
	gint                   slide_step;
	
	GList                 *custom_mime_apps;
	
	gboolean               open_dirs;
	
	char*                  text_color;
	gboolean               text_italic;
};

struct _KibaStack {
	GObject           parent;
	
	KibaStackPlugin  *stack_plugin; /* the KibaStackPlugin instance,
					   the pointer is the same for each stack */
	
	GtkWidget        *widget;  /* the object on the dock representing this stack */
	
	GList            *icons;   /* list with all KibaStackIcon's */
	gint              count;
	
	char             *uri;  /* stack uri (usually ~/.kiba-dock/stack/*.stack) */
};

struct _KibaStackClass {
	GObjectClass parent_class;
	
	void (* icon_added) (KibaStack    *stack,
			     KibaStackIcon *icon);
	void (* icon_removed) (KibaStack    *laucher,
			       KibaStackIcon *icon);
};

KibaStack* kiba_stack_new (KibaStackPlugin *plugin,
			   const char      *uri);

const char* kiba_stack_get_custom_command_for_mime_type (KibaStackPlugin *plugin,
							 const char *mime_type);

void kiba_stack_set_custom_mime_app (KibaStackPlugin *plugin,
				     const char *mime_type,
				     const char *command);

void kiba_stack_add (KibaStack    *stack,
		     KibaStackIcon *icon);

void kiba_stack_remove (KibaStack    *stack,
			KibaStackIcon *icon);

cairo_surface_t* kiba_stack_get_mime_surface (KibaStack  *stack,
					      const char *mime_type,
					      const char *mime_icon);

void kiba_stack_load_thumb_for_icon (KibaStackIcon *icon);

void kiba_stack_destroy (KibaStack *stack);

void kiba_stack_toggle_icon_view (KibaStack *stack);

#endif
